/*-
 * Copyright (c) 2015 Weongyo Jeong <weongyo@3rocks.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

var g_conf_pac_config_default = `function FindProxyForURL(url, host) {
  host = host.toLowerCase();
  if (isPlainHostName(host) ||
      isInNetEx(host, "127.0.0.0/8") ||
      isInNetEx(host, "10.0.0.0/8") ||
      isInNetEx(host, "172.16.0.0/12") ||
      isInNetEx(host, "192.168.0.0/16") ||
      isInNetEx(host, "fc00::/7") ||
      isInNetEx(host, "fe80::/10") ||
      shExpMatch(url, "http://mudfish.net/*") ||
      shExpMatch(url, "https://mudfish.net/*")) {
        return "DIRECT";
  }
  return "{0}";
}`;

var g_conf = {
  check_current_ip: true,
  refresh_active_tab: false,
  use_secure_connection: false,
  use_ssl_bump: false,
  remember_me: false,
  username: "",
  password: "",
  pac_config: g_conf_pac_config_default
};

function send_auth() {
  chrome.runtime.sendMessage({
    type: 'SET_AUTH',
    username: g_conf.username,
    password: g_conf.password,
  }, function (response) {
    if (response.error_code != 0) {
      console.log("Failed to send 'SET_AUTH' command.");
    }
  });
}

function send_pac_config() {
  chrome.runtime.sendMessage({
    type: 'SET_PAC_CONFIG',
    pac_config: g_conf.pac_config
  }, function (response) {
    if (response.error_code != 0) {
      console.log("Failed to send 'SET_PAC_CONFIG' command.");
    }
  });
}

function get_staticnodes_callback(data) {
  var html = "";

  if (data.staticnodes == null || data.staticnodes == undefined)
    return (false);

  html = html + "<select id='staticnodes_select' style='width:246px'>";
  html = html + "<option></option>";
  for (i = 0; i < data.staticnodes.length; i++) {
    html = html + "<option value='" + data.staticnodes[i].sid + "'>";
    html = html + data.staticnodes[i].location;
    html = html + "</option>";
  }
  html = html + "</select>";
  $("#staticnodes_list").html(html);
  $("#staticnodes_select").change(function () {
    chrome.runtime.sendMessage({
      type: 'CONNECT',
      sid: $("#staticnodes_select").val(),
      use_secure_connection: g_conf.use_secure_connection,
      use_ssl_bump: g_conf.use_ssl_bump
    }, function (resp) {
      if (resp.error_code == 0) {
        var location, sid;

        location = $("#staticnodes_select option:selected").text();
        sid = $("#staticnodes_select").val();
        add_recently_used(location, sid);

        chrome.action.setIcon({
          path: "/images/appIcon/default-icon.png"
        });
        chrome.action.setPopup({
          popup: "/html/popup_disconnect.html"
        });
        if (g_conf.refresh_active_tab) {
          var queryInfo = { active: true, currentWindow: true };
          chrome.tabs.query(queryInfo, function (arrayOfTabs) {
            var code = 'window.location.reload();';
            chrome.tabs.executeScript(arrayOfTabs[0].id, { code: code });
          });
        }
        if (g_conf.check_current_ip) {
          setTimeout(function () {
            chrome.tabs.create({ url: "https://myip.mudfish.net" });
          }, 50);
        } else {
          window.close();
        }
      } else {
        alert(resp.error_msg);
      }
    });
  });
  $("#staticnodes_select").select2({
    placeholder: chrome.i18n.getMessage("popup_connect_select_node")
  }).on("select2:open", function (e) {
    $("#td_staticnodes_list").css("padding-bottom", "200px");
  }).on("select2:close", function (e) {
    $("#td_staticnodes_list").css("padding-bottom", "0px");
  });
  return (true);
}

function get_staticnodes() {

  chrome.runtime.sendMessage({
    type: 'GET_STATICNODES'
  }, function (response) {
    if (!get_staticnodes_callback(response)) {
      setTimeout(function () { get_staticnodes(); }, 1000);
    }
  });
}

function load_recently_used_callback(recently_used) {
  var html = "";

  html = html + "<select id='recently_used_select' style='width:246px;'>";
  html = html + "<option></option>";
  for (i = 0; i < recently_used.length; i++) {
    html = html + "<option value='" + recently_used[i].sid + "'>";
    html = html + recently_used[i].location;
    html = html + "</option>";
  }
  html = html + "</select>";
  $("#recently_used_list").html(html);
  $("#recently_used_select").change(function () {
    chrome.runtime.sendMessage({
      type: 'CONNECT',
      sid: $("#recently_used_select").val(),
      use_secure_connection: g_conf.use_secure_connection,
      use_ssl_bump: g_conf.use_ssl_bump
    }, function (resp) {
      if (resp.error_code == 0) {
        chrome.action.setIcon({
          path: "/images/appIcon/default-icon.png"
        });
        chrome.action.setPopup({
          popup: "/html/popup_disconnect.html"
        });
        if (g_conf.refresh_active_tab) {
          var queryInfo = { active: true, currentWindow: true };
          chrome.tabs.query(queryInfo, function (arrayOfTabs) {
            var code = 'window.location.reload();';
            chrome.tabs.executeScript(arrayOfTabs[0].id, { code: code });
          });
        }
        if (g_conf.check_current_ip) {
          setTimeout(function () {
            chrome.tabs.create({ url: "https://myip.mudfish.net" });
          }, 50);
        } else {
          window.close();
        }
      } else {
        alert(resp.error_msg);
      }
    });
  });
  $("#recently_used_select").select2({
    placeholder: chrome.i18n.getMessage("popup_connect_recently_used_node")
  }).on("select2:open", function (e) {
    $("#td_recently_used_list").css("padding-bottom", "200px");
  }).on("select2:close", function (e) {
    $("#td_recently_used_list").css("padding-bottom", "0px");
  });
}

var g_recently_used = [];
var g_recently_used_enabled = false;

function save_recently_used() {

  chrome.storage.sync.set({
    "recently_used": g_recently_used,
    "recently_used_enabled": g_recently_used_enabled
  }, function () {
    load_recently_used_callback(g_recently_used);
  });
}

function clear_recently_used() {

  g_recently_used = [];
  chrome.storage.sync.set({
    "recently_used": g_recently_used,
    "recently_used_enabled": g_recently_used_enabled
  }, function () {
    load_recently_used_callback(g_recently_used);
  });
}

function add_recently_used(location, sid) {
  var staticnode = { location: location, sid: sid };
  var recently_used = [];

  for (var i = 0; i < g_recently_used.length; i++) {
    if (g_recently_used[i].sid == sid) {
      continue;
    }
    recently_used.push(g_recently_used[i]);
  }
  recently_used.unshift(staticnode);
  g_recently_used = recently_used;
  save_recently_used();
}

function load_recently_used() {
  chrome.storage.sync.get({
    "recently_used": [],
    "recently_used_enabled": false
  }, function (items) {
    load_recently_used_callback(items.recently_used);
    g_recently_used = items.recently_used;
    if (items.recently_used_enabled)
      change_to_recently_used_menu();
  });
}

function change_to_recently_used_menu() {
  var display;

  display = $("#tr_recently_used_list").css("display");
  if (display == "none") {
    $("#mudfish_conf_recently_used").css("color", "black");
    if (g_recently_used.length > 0)
      $("#mudfish_conf_recently_used_cleaner").css("display", "");
    $("#tr_staticnodes_list").css("display", "none");
    $("#tr_recently_used_list").css("display", "block");
    g_recently_used_enabled = true;
  } else {
    $("#mudfish_conf_recently_used").css("color", "gray");
    $("#mudfish_conf_recently_used_cleaner").css("display", "none");
    $("#tr_staticnodes_list").css("display", "block");
    $("#tr_recently_used_list").css("display", "none");
    g_recently_used_enabled = false;
    get_staticnodes();
  }
  save_recently_used();
}

function clear_recently_used_list() {
  clear_recently_used();
  $("#mudfish_conf_recently_used_cleaner").css("display", "none");
}

function handle_recently_used() {

  load_recently_used();
  $("#mudfish_conf_recently_used").click(change_to_recently_used_menu);
  $("#mudfish_conf_recently_used_cleaner").click(clear_recently_used_list);
}

var g_syncing_staticNodes = false;

function handle_sync_staticnodes() {

  $("#mudfish_conf_refresh_staticnodes").click(function () {
    $("#mudfish_conf_refresh_staticnodes").addClass("fa-spin");
    $("#mudfish_conf_msg").html(chrome.i18n.getMessage("popup_connect_syncing"));
    g_syncing_staticNodes = true;
    chrome.runtime.sendMessage({
      type: 'SYNC_STATICNODES'
    }, function (resp) {
      $("#mudfish_conf_refresh_staticnodes").removeClass("fa-spin");
      g_syncing_staticNodes = false;
      if (resp.error_code == 0) {
        var l_synced = chrome.i18n.getMessage("popup_connect_synced");

        get_staticnodes();
        $("#mudfish_conf_msg").html(l_synced);
        window.setTimeout(function () {
          $("#mudfish_conf_msg").html("");
        }, 500);
      } else {
        $("#mudfish_conf_msg").html(resp.error_msg);
      }
    });
  });
}

function handle_options() {
  chrome.storage.sync.get(g_conf, function (items) {
    g_conf = items;
    send_auth();
    send_pac_config();
  });
  $("#mudfish_conf_options").click(function () {
    if (chrome.runtime.openOptionsPage) {
      chrome.runtime.openOptionsPage();
    } else {
      window.open(chrome.runtime.getURL('html/options.html'));
    }
  });
}

function popup_connect_i18n_html(name) {
  var m;

  m = chrome.i18n.getMessage(name);
  $("#" + name).html(m);
}

function popup_connect_i18n_attr(name, attr) {
  var m;

  m = chrome.i18n.getMessage(name);
  $("#" + name).attr(attr, m);
}

function popup_connect_i18n() {
  var m;

  m = chrome.i18n.getMessage("popup_connect_loading");
  $("#staticnodes_list").html(m);
  $("#recently_used_list").html(m);
  popup_connect_i18n_attr("mudfish_conf_recently_used", "title");
  popup_connect_i18n_attr("popup_connect_homepage", "title");
  popup_connect_i18n_attr("mudfish_conf_refresh_staticnodes", "title");
  popup_connect_i18n_attr("mudfish_conf_options", "title");
}

$(document).ready(function () {
  popup_connect_i18n();
  handle_recently_used();
  handle_options();
  handle_sync_staticnodes();
  get_staticnodes();
});
